import pygame
import sys
import math
from random import randint
from pygame.locals import *

WIDTH = 800
HEIGHT = 600

BAT_SIZE = 128

ROWS_OF_BRICKS = 7
DEFAULT_FPS = 60

BRICK_VERTICAL_OFFSET = 64
TOP_OFFSET = 32
BRICK_PADDING = 2
BRICK_SCORE = 10

BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
BLUE = (0, 0, 255)
GREEN = (0, 255, 0)
RED = (255, 0, 0)
YELLOW = (255, 216, 0)

#setup...
pygame.mixer.init(frequency=22050, size=-16, channels=2, buffer=512)
pygame.init()
clock = pygame.time.Clock()
screen = pygame.display.set_mode([WIDTH, HEIGHT])
screen.fill(BLACK)
pygame.mouse.set_visible(0)
fps = DEFAULT_FPS

wall = [[]]

#resources...
font = pygame.font.SysFont("monospace", 15)
brick_green = pygame.image.load("brick_green.png")          
brick_blue = pygame.image.load("brick_blue.png")       
brick_yellow = pygame.image.load("brick_yellow.png")          
brick_orange = pygame.image.load("brick_orange.png")           
brick_red = pygame.image.load("brick_red.png")
    
# load the ball image
ball = pygame.image.load("ball.png")
ball_rect = ball.get_rect()
ball_width = ball_rect.width
ball_height = ball_rect.height
ball_rect.x = 100
ball_rect.y = 200
ball_style = "NORMAL"
ball_style_countdown = 0

# game
DEFAULT_LIVES = 3
score = 0
lives = DEFAULT_LIVES
    
# lives ball
lives_ball = pygame.image.load("ball.png")
lives_ball_rect = ball.get_rect()
lives_ball_rect.x = 0
lives_ball_rect.y = 8

# velocity
DEFAULT_VX = 4 
DEFAULT_VY = 8
ball_vx = 0
ball_vy = 0

# bat position
bat_x = WIDTH / 2 - (BAT_SIZE / 2)
bat_y = HEIGHT - 16

# load sound...
pygame.mixer.set_num_channels(10)
voice = pygame.mixer.Channel(8)

sound_bat = pygame.mixer.Sound("bat_hit.wav")
sound_brick = pygame.mixer.Sound("brick_hit.wav")


# roughly figure out how many columns will fit on screen
dummy_brick = pygame.image.load("brick_green.png")
brick_width = dummy_brick.get_rect().width
brick_height = dummy_brick.get_rect().height

columns = round(WIDTH / dummy_brick.get_rect().width)

pygame.mixer.music.load("music.mp3")
pygame.mixer.music.play(-1)

remaining_bricks = ROWS_OF_BRICKS * columns

def reset_wall():
    global wall
    wall = [
        [randint(1, 5) for x in range(columns)]
        for y in range(ROWS_OF_BRICKS)]

def draw_wall():
    global wall
    for row in range(len(wall)):
        for col in range(len(wall[row])):
            
            brick = None
            if wall[row][col] != 0:
                val = randint(1,5)
            else:
                val = 0

            if val == 1:
                brick = pygame.image.load("brick_green.png")
            elif val == 2:
                brick = pygame.image.load("brick_blue.png")
            elif val == 3:
                brick = pygame.image.load("brick_yellow.png")
            elif val == 4:
                brick = pygame.image.load("brick_orange.png")
            elif val == 5:
                brick = pygame.image.load("brick_red.png")

            if brick is not None:
                brick_rect = brick.get_rect()

                brick_rect.x = col * (brick_width + BRICK_PADDING)
                brick_rect.y = BRICK_VERTICAL_OFFSET + (row * (brick_height + BRICK_PADDING))

                screen.blit(brick, brick_rect)

def draw_wall_pi():
    global wall, brick_green, brick_blue, brick_yellow, brick_orange, brick_red
    for row in range(len(wall)):
        for col in range(len(wall[row])):
            
            brick = None
            if wall[row][col] != 0:
                val = randint(1,5)
            else:
                val = 0
            if val == 1:
                brick = brick_green
            elif val == 2:
                brick = brick_blue
            elif val == 3:
                brick = brick_yellow
            elif val == 4:
                brick = brick_orange
            elif val == 5:
                brick = brick_red

            if brick is not None:
                brick_rect = brick.get_rect()

                brick_rect.x = col * (brick_width + BRICK_PADDING)
                brick_rect.y = BRICK_VERTICAL_OFFSET + (row * (brick_height + BRICK_PADDING))

                screen.blit(brick, brick_rect)

def reset_ball():
    global bat_x, ball_vx, ball_vy, ball_rect
    bat_x = WIDTH / 2 - (BAT_SIZE / 2)
    ball_vx = 0
    ball_vy = 0
    ball_rect.x = bat_x + (BAT_SIZE / 2 - ball_width / 2)
    ball_rect.y = bat_y - ball_height

def set_ball(style, countdown):
    global ball, ball_rect, ball_width
    global ball_height, ball, ball_style, ball_style_countdown

    ball_style_countdown = countdown
    x = ball_rect.x
    y = ball_rect.y
    
    if style == "NORMAL":
        ball = pygame.image.load("ball.png")
        
    elif style == "GOSH":
        ball = pygame.image.load("ball_gosh.png")

    elif style == "SMILE":
        ball = pygame.image.load("ball_smile.png")

    ball_rect = ball.get_rect()
    ball_rect.x = x
    ball_rect.y = y
    ball_width = ball_rect.width
    ball_height = ball_rect.height
    ball_style = style

def play_ball_groan():
    global sound_hit1, sound_hit2, sound_hit3, sound_hit4

    i = randint(1, 4)

    if i == 1:
        sound_hit1.play()
    elif i == 2:
        sound_hit2.play()
    elif i == 3:
        sound_hit3.play()
    elif i == 4:
        sound_hit4.play()

if __name__ == "__main__":
    reset_wall()
    
    while 1:
        pygame.event.pump()
        clock.tick(fps) #frames per second

        # capture keyboard input and update the game
        keys = pygame.key.get_pressed()
        x, y = pygame.mouse.get_pos()
        
        if keys[pygame.K_ESCAPE] or keys[pygame.K_q]:
            pygame.quit()
            sys.exit(0)

        for event in pygame.event.get():
            if event.type == MOUSEMOTION:
                x, y = event.pos    
                bat_x = x

            if event.type == MOUSEBUTTONUP and lives > 0 and ball_vy == 0:
                ball_vx = DEFAULT_VX
                ball_vy = DEFAULT_VY

        if keys[pygame.K_LEFT]:
            bat_x -= 16

        elif keys[pygame.K_RIGHT]:
            bat_x += 16

        elif keys[pygame.K_SPACE]:
            if lives > 0 and ball_vy == 0:
                ball_vx = DEFAULT_VX
                ball_vy = DEFAULT_VY

        elif keys[pygame.K_r]:
            if lives == 0:
                reset_ball()
                lives = DEFAULT_LIVES
                score = 0
                fps = DEFAULT_FPS
                remaining_bricks = ROWS_OF_BRICKS * columns
                reset_wall()

        # collision with top
        if ball_vy < 0 and ball_rect.y < TOP_OFFSET :
            ball_vy *= -1

        # off screen
        if ball_vy > 0 and ball_rect.y > HEIGHT + 16:
            lives -= 1
            reset_ball()

        # bat colltion...
        elif ball_vy > 0 and ball_rect.y + ball_height >= bat_y \
                and (ball_rect.x + (ball_width / 2) > bat_x - (ball_width / 2)
                     and ball_rect.x < bat_x + BAT_SIZE + (ball_width / 2)):
            ball_vy *= -1
            bat_middle = bat_x + (BAT_SIZE / 2)

            if ball_rect.x > bat_middle:
                ball_vx += (0.05 * (ball_rect.x - bat_middle))
                ball_vx = min(ball_vx, 4)
            else:
                ball_vx -= (0.05 * (bat_middle - ball_rect.x))
                ball_vx = max(ball_vx, -4)

            set_ball("SMILE", 10)
            sound_bat.play()

        if ball_rect.x + ball_width > WIDTH or ball_rect.x <= 0:
            ball_vx *= -1
            
        # work out if the ball has hit a brick
        ball_col = round(((ball_rect.x + (ball_width / 2)) // (brick_width + 2)))
        ball_row = round(((ball_rect.y - (BRICK_VERTICAL_OFFSET - (ball_width / 2))) // (brick_height + 2)))

        block_value = 0

        # if (ball_row >= 0 and ball_row <= (ROWS_OF_BRICKS - 1)) \
        #         and (ball_col >= 0 and ball_col <= (columns - 1)):

        if (0 <= ball_row <= (ROWS_OF_BRICKS - 1)) \
                and (0 <= ball_col <= (columns - 1)):
            block_value = wall[ball_row][ball_col]
            
        if block_value != 0:
            wall[ball_row][ball_col] = 0
            ball_vy *= -1
            score += BRICK_SCORE
            remaining_bricks -= 1
            set_ball("GOSH", 10)
            
            sound_brick.play()
            #play_ball_groan()


        #If the ball has stopped moving, put it on the bat...
        if ball_vx == 0:
            ball_rect.x = bat_x + (BAT_SIZE / 2 - ball_width / 2)
            ball_rect.y = bat_y - ball_height

        # move the ball normally
        else:
            ball_rect.x = ball_rect.x + ball_vx
            ball_rect.y = ball_rect.y + ball_vy

        # ************** Draw stuff ****************
        # black out the screen.
        screen.fill(BLACK)

        # draw the wall...
        draw_wall_pi()

        # draw the ball
        screen.blit(ball, ball_rect)
        
        # draw the bat using a simple line
        pygame.draw.line(screen, WHITE, [bat_x, bat_y], [bat_x + BAT_SIZE, HEIGHT - 16], 8)

        # draw the score
        label = font.render("SCORE: " + str(int(score)), 1, WHITE)
        screen.blit(label, (8, 8))

        # draw top edge
        pygame.draw.line(screen, WHITE, [0, TOP_OFFSET], [WIDTH, TOP_OFFSET], 1)

        for l in range(lives):
            lives_ball_rect.x = (WIDTH - 8) - (ball_width * lives) + (l * ball_width)
            screen.blit(lives_ball, lives_ball_rect)

        if lives == 0:
            label = font.render("NO MORE LIVES!!!", 1, RED)
            screen.blit(label, (WIDTH / 2 - (label.get_width() / 2), HEIGHT / 2))

            last_label_y = HEIGHT / 2
            label = font.render("Score - " + str(score), 1, YELLOW)
            screen.blit(label, (WIDTH / 2 - (label.get_width() / 2), last_label_y + label.get_height()))

            last_label_y = last_label_y + label.get_height()        
            label = font.render("(Hit R to restart)", 1, WHITE)
            screen.blit(label, (WIDTH / 2 - (label.get_width() / 2), last_label_y + label.get_height()))

        # render everything in the buffer...
        pygame.display.flip()

        if remaining_bricks == 0 :
            reset_ball()
            fps *= 1.10
            remaining_bricks = ROWS_OF_BRICKS * columns 
            reset_wall()
            
        if ball_style_countdown == 0 and ball_style != "NORMAL":
            set_ball ("NORMAL", 0)
        else:
            ball_style_countdown -= 1



